#!/usr/bin/sudo bash

DEBUG=false
TABLE_ID=2000
PRIORITY=2000
IGNORE_IP=""

while getopts ":dt:p:i:" opt; do
    case ${opt} in
        d )
            DEBUG=true
            ;;
        t )
            TABLE_ID=$OPTARG
            ;;
        p )
            PRIORITY=$OPTARG
            ;;
        i )
            IGNORE_IP=$OPTARG
            ;;

        \? )
            echo "Invalid option: $OPTARG" 1>&2
            ;;
        : )
            echo "Invalid option: $OPTARG requires an argument" 1>&2
            ;;
    esac
done




INTERFACES=$(ip -o link | awk -F ":" '{print $2}' | grep -vw lo | grep -v \@ | tr '\n' ' ' | awk '{$1=$1};1' )
MAIN_INTERFACE=$(ip -o route get 8.8.8.8 | grep -oP "dev \K[\w]+")

if [ "$DEBUG" = true ] ; then
    echo "TABLE_ID = \"$TABLE_ID\""
    echo "INTERFACES = \"$INTERFACES\""
    echo "MAIN_INTERFACE = \"$MAIN_INTERFACE\""
    echo "IGNORE_IP = \"$IGNORE_IP\""
    echo
fi

while ip rule del pref $PRIORITY 2>/dev/null ; do true ; done

for INTERFACE in $INTERFACES ; {
    if [ "$INTERFACE" = "$MAIN_INTERFACE" ] ; then
        continue
    fi
    for IP in $(ip -f inet addr show $INTERFACE | grep -v vpn | grep -oP "inet \K[\d.]+") ; {
        if [[ "$IGNORE_IP" == *"$IP"* ]] ; then
            if [ "$DEBUG" = true ] ; then echo "skipping private IP ($IP)" ; fi
            continue;
        fi
        if [ "$DEBUG" = true ] ; then
            echo "Considering $INTERFACE($IP)"
        fi
        # Use the route the vendor provided for routing from this interface.
        GW=$(ip route | grep -o "via .* dev $INTERFACE" -m 1 | awk ' { print $2 } ')
        ip route flush table $TABLE_ID || true 
        ip rule add from $IP table $TABLE_ID pref $PRIORITY
        ip route add default via $GW dev $INTERFACE table $TABLE_ID
        TABLE_ID=$(( $TABLE_ID + 1 ))
    }
}
